Unit CrypUnit;
Interface
uses
  VPUtils,SysUtils, Classes,
  sha256, crypt2, ciphers,blowfish,Base64,dom,xmlread,xmlwrite;

procedure EncryptClick(InputName,OutputName,Keyword:string;
                        Cipher:TDCP_blowfish);

procedure DecryptClick(InputName,OutputName,Keyword:string;
                        Cipher:TDCP_blowfish);

procedure EncryptStream(InputStream:TMemoryStream;
                        OutputStream:TFileStream;
                        Keyword:string;
                        Cipher:TDCP_blowfish);
procedure DecryptStream(InputStream:TFileStream;
                        OutputStream:TMemoryStream;
                        Keyword:string;
                        Cipher:TDCP_blowfish);
implementation

function Min(a, b: integer): integer;
begin
  if (a < b) then
    Result := a
  else
    Result := b;
end;

procedure EncryptClick(InputName,OutputName,Keyword:string;
                        Cipher:TDCP_blowfish);
var
  CipherIV: array[0..7] of byte;     // the initialisation vector (for chaining modes)
  Hash256: TDCP_sha256;             // the hash to use
  HashDigest: array[0..31] of byte;   // the result of hashing the passphrase with the salt
  Salt: array[0..7] of byte;   // a random salt to help prevent precomputated attacks
  strmInput, strmOutput: TFileStream;
  i: integer;
  Work:pChar;
begin
  strmInput := nil;
  strmOutput := nil;
  try
    strmInput := TFileStream.Create(InputName,fmOpenRead);
    strmOutput := TFileStream.Create(OutputName,fmCreate);

    Hash256 := TDCP_sha256.Create;
    for i := 0 to 7 do
      Salt[i] := Random(256);
       // just fill the salt with random values (crypto secure PRNG would be better but not _really_ necessary)
    strmOutput.WriteBuffer(Salt,Sizeof(Salt));
     // write out the salt so we can decrypt!
    Hash256.Init;
    Hash256.Update(Salt[0],Sizeof(Salt));   // hash the salt
    Hash256.UpdateStr(keyword);  // and the passphrase
    Hash256.Final(HashDigest[0]);           // store the output in HashDigest

    // if the cipher is a block cipher we need an initialisation vector
    begin
      for i := 0 to (SizeOf(CipherIV) - 1) do
        CipherIV[i] := Random(256);           // again just random values for the IV
      strmOutput.WriteBuffer(CipherIV[0],SizeOf(CipherIV));  // write out the IV so we can decrypt!
      Cipher.Init(HashDigest[0],
                  Min(Cipher.GetMaxKeySize,
                  Hash256.GetHashSize),
                  @CipherIV[0]);
      // initialise the cipher with the hash as key
      TDCP_blockcipher(Cipher).CipherMode := cmCBC;   // use CBC chaining when encrypting
    end;

    Cipher.EncryptStream(strmInput,strmOutput,strmInput.Size); // encrypt the entire file
    Cipher.Burn;   // important! get rid of keying information
    strmInput.Free;
    strmOutput.Free;
    hash256.free;
  except
    strmInput.Free;
    strmOutput.Free;
    hash256.free;
  end;
end;

procedure DecryptClick(InputName,OutputName,Keyword:string;
                        Cipher:TDCP_blowfish);
var
  CipherIV: array[0..7] of byte;     // the initialisation vector (for chaining modes)
  Hash256: TDCP_sha256;             // the hash to use
  HashDigest: array[0..31] of byte;   // the result of hashing the passphrase with the salt
  Salt: array[0..7] of byte;   // a random salt to help prevent precomputated attacks
  strmInput, strmOutput: TFileStream;
begin
  strmInput := nil;
  strmOutput := nil;
  try
    strmInput := TFileStream.Create(InputName,fmOpenRead);
    strmOutput := TFileStream.Create(OutputName,fmCreate);

    Hash256 := TDCP_sha256.Create;
    strmInput.ReadBuffer(Salt[0],Sizeof(Salt));  // read the salt in from the file
    Hash256.Init;
    Hash256.Update(Salt[0],Sizeof(Salt));   // hash the salt
    Hash256.UpdateStr(keyword);  // and the passphrase
    Hash256.Final(HashDigest[0]);           // store the hash in HashDigest

    // if it is a block cipher we need the IV
    begin
      strmInput.ReadBuffer(CipherIV[0],SizeOf(CipherIV));
      // read the initialisation vector from the file
      Cipher.Init(HashDigest[0],
                  Min(Cipher.GetMaxKeySize,
                  Hash256.GetHashSize),
                  @CipherIV);  // initialise the cipher
      TDCP_blockcipher(Cipher).CipherMode := cmCBC;
    end;

    Cipher.DecryptStream(strmInput,strmOutput,strmInput.Size - strmInput.Position); // decrypt!
    Cipher.Burn;
    strmInput.Free;
    strmOutput.Free;
    hash256.free;
  except
    strmInput.Free;
    strmOutput.Free;
    hash256.free;
  end;
end;

procedure EncryptStream(InputStream:TMemoryStream;
                        OutputStream:TFileStream;
                        Keyword:string;
                        Cipher:TDCP_blowfish);
var
  CipherIV: array[0..7] of byte;     // the initialisation vector (for chaining modes)
  Hash256: TDCP_sha256;             // the hash to use
  HashDigest: array[0..31] of byte;   // the result of hashing the passphrase with the salt
  Salt: array[0..7] of byte;   // a random salt to help prevent precomputated attacks
  strmOutput: TFileStream;
  i: integer;
  Work:pChar;
begin
  strmOutput := nil;
  try
    strmOutput := OutputStream;

    Hash256 := TDCP_sha256.Create;
    for i := 0 to 7 do
      Salt[i] := Random(256);
       // just fill the salt with random values (crypto secure PRNG would be better but not _really_ necessary)
    strmOutput.WriteBuffer(Salt,Sizeof(Salt));
     // write out the salt so we can decrypt!
    Hash256.Init;
    Hash256.Update(Salt[0],Sizeof(Salt));   // hash the salt
    Hash256.UpdateStr(keyword);  // and the passphrase
    Hash256.Final(HashDigest[0]);           // store the output in HashDigest

    // if the cipher is a block cipher we need an initialisation vector
    begin
      for i := 0 to (SizeOf(CipherIV) - 1) do
        CipherIV[i] := Random(256);           // again just random values for the IV
      strmOutput.WriteBuffer(CipherIV[0],SizeOf(CipherIV));  // write out the IV so we can decrypt!
      Cipher.Init(HashDigest[0],
                  Min(Cipher.GetMaxKeySize,
                  Hash256.GetHashSize),
                  @CipherIV[0]);
      // initialise the cipher with the hash as key
      TDCP_blockcipher(Cipher).CipherMode := cmCBC;   // use CBC chaining when encrypting
    end;

    Cipher.EncryptStream(InputStream,strmOutput,InputStream.Size); // encrypt the entire file
    Cipher.Burn;   // important! get rid of keying information
    strmOutput.Free;
    hash256.free;
  except
    strmOutput.Free;
    hash256.free;
  end;
end;

procedure DecryptStream(InputStream:TFileStream;
                        OutputStream:TMemoryStream;
                        Keyword:string;
                        Cipher:TDCP_blowfish);
var
  CipherIV: array[0..7] of byte;     // the initialisation vector (for chaining modes)
  Hash256: TDCP_sha256;             // the hash to use
  HashDigest: array[0..31] of byte;   // the result of hashing the passphrase with the salt
  Salt: array[0..7] of byte;   // a random salt to help prevent precomputated attacks
  strmInput,FileStream: TFileStream;
begin
  strmInput := nil;
  try
    strmInput := InputStream;

    Hash256 := TDCP_sha256.Create;
    strmInput.ReadBuffer(Salt[0],Sizeof(Salt));  // read the salt in from the file
    Hash256.Init;
    Hash256.Update(Salt[0],Sizeof(Salt));   // hash the salt
    Hash256.UpdateStr(keyword);  // and the passphrase
    Hash256.Final(HashDigest[0]);           // store the hash in HashDigest

    // if it is a block cipher we need the IV
    begin
      strmInput.ReadBuffer(CipherIV[0],SizeOf(CipherIV));
      // read the initialisation vector from the file
      Cipher.Init(HashDigest[0],
                  Min(Cipher.GetMaxKeySize,
                  Hash256.GetHashSize),
                  @CipherIV);  // initialise the cipher
      TDCP_blockcipher(Cipher).CipherMode := cmCBC;
    end;

    Cipher.DecryptStream(strmInput,OutputStream,strmInput.Size - strmInput.Position); // decrypt!
    Cipher.Burn;
    strmInput.Free;
    hash256.free;
  except
    hash256.free;
    strmInput.Free;
  end;
end;

begin
end.

var
    blow:TDCP_Blowfish;
    MemStream:TMemoryStream;
    Doc:TXMLDocument;
begin
    blow:=tdcp_blowfish.create;
    blow.selftest;

    EncryptClick('test.xml','test.enc','Keyword',blow);
    DecryptClick('test.enc','test.xo1','Keyword',blow);

    MemStream:=TMemoryStream.Create;
    DecryptStream('test.enc',MemStream,'Keyword',blow);
    memStream.position:=0;
    ReadXMLFileStream(Doc,TStream(MemStream));
    MemStream.free;
    WriteXMLFileName(Doc,'test.xo2');


    MemStream:=TMemoryStream.Create;
    WriteXMLFileStream(Doc,TStream(MemStream));
    MemStream.position:=0;
    EncryptStream(MemStream,'test.xo3','Keyword',blow);
    DecryptClick('test.xo3','test.xm3','Keyword',blow);
end.



















